""" 

"""

import sys
import time

import nealParams as nealParameters
#get the simulator
if (nealParameters.simulator=="spinnaker"):
    import pyNN.spiNNaker as sim
    import spynnaker_external_devices_plugin.pyNN as ExternalDevices

elif (nealParameters.simulator=="nest"):
    import pyNN.nest as sim
    import cPickle as pickle


from nealCoverClass import NealCoverFunctions
from parseClass import ParseClass
import turnStepLang as languageToDefine

def init():
    #print "spin" or nest
    sim.setup(timestep=nealParameters.DELAY,
                    min_delay=nealParameters.DELAY,
                    max_delay=nealParameters.DELAY, debug=0)

def createInputs(startTime):
    inputSpikeTimes0 = [10.0 + startTime] 
    inputSpikeTimes1 = [50.0 + startTime]
    inputSpikeTimes2 = [100.0 + startTime]
    inputSpikeTimes3 = [150.0 + startTime]
    inputSpikeTimes4 = [200.0 + startTime]
    inputSpikeTimes5 = [250.0 + startTime]
    inputSpikeTimes6 = [300.0 + startTime]
    stopSpikeTimes = [400.0 + startTime]
    
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeArray2 = {'spike_times': [inputSpikeTimes2]}
    spikeArray3 = {'spike_times': [inputSpikeTimes3]}
    spikeArray4 = {'spike_times': [inputSpikeTimes4]}
    spikeArray5 = {'spike_times': [inputSpikeTimes5]}
    spikeArray6 = {'spike_times': [inputSpikeTimes6]}
    stopSpikeArray = {'spike_times': [stopSpikeTimes]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    spikeGen1=sim.Population(1, sim.SpikeSourceArray, spikeArray1,
                                   label='inputSpikes_1')
    spikeGen2=sim.Population(1, sim.SpikeSourceArray, spikeArray2,
                                   label='inputSpikes_2')
    spikeGen3=sim.Population(1, sim.SpikeSourceArray, spikeArray3,
                                   label='inputSpikes_3')
    spikeGen4=sim.Population(1, sim.SpikeSourceArray, spikeArray4,
                                   label='inputSpikes_4')
    spikeGen5=sim.Population(1, sim.SpikeSourceArray, spikeArray5,
                                   label='inputSpikes_5')
    spikeGen6=sim.Population(1, sim.SpikeSourceArray, spikeArray6,
                                   label='inputSpikes_6')
    stopSpikeGen=sim.Population(1, sim.SpikeSourceArray, stopSpikeArray,
                                   label='stopSpikes')

    return [spikeGen0,spikeGen1,spikeGen2,spikeGen3,spikeGen4,spikeGen5,
            spikeGen6,stopSpikeGen]

def linkInSink(spikeSource,sinkCell):
    if (nealParameters.simulator == 'spinnaker'):
        connector = [(0,0,1.0, nealParameters.DELAY)]
        neal.nealProjection(spikeSource,sinkCell,connector,'inhibitory') 

#spinnaker crashes if a spike source isn't connected to
#something.  Make the something here.
def allocateSinks(firstSink,generators):
    if (nealParameters.simulator != 'spinnaker'):
        return
    sinkCell=sim.Population(1,sim.IF_cond_exp,{})
    for sinkNumber  in range (firstSink,7):
        linkInSink(spikeGenerators[sinkNumber],sinkCell)

def setNestInput(currentSimulationTime,parser):
    parser.setNestInputs(currentSimulationTime)

def newActionOutput(currentSimulationTime,actionCells):
    if (nealParameters.simulator=="nest"):
        output = []
        actionData = actionCells.get_data()
        actionSegments = actionData.segments[0]
        actionSpikeTrains = actionSegments.spiketrains
        for action in range (0,len(actionSpikeTrains)):
            if (len(actionSpikeTrains[action]) > 0):
                anActionsSpikes = actionSpikeTrains[action]
                for spike in range (0,len(anActionsSpikes)):
                    aSpike = anActionsSpikes[spike]
                    if (aSpike <= currentSimulationTime) and (aSpike > currentSimulationTime-30):
                        output = output + [action]
                        print 'ho', currentSimulationTime,action

        return output
    return []

def setNestOutput(currentSimulationTime,plan):
    if (currentSimulationTime == 0):
        nestOutputFileHandle = open("world/actions.txt",'a')
        nestOutputFileHandle.write(str(currentSimulationTime)+"\n")
        nestOutputFileHandle.close()
        return
    actions = newActionOutput(currentSimulationTime,plan) 
    if (len(actions) > 0):
        nestOutputFileHandle = open("world/actions.txt",'a')
        for actionNum in range (0,len(actions)):
            nestOutputFileHandle.write(str(actions[actionNum])+" "+
                                       str(currentSimulationTime)+"\n")
        nestOutputFileHandle.close()

def runAgent(parser,actionCells):
    if (nealParameters.simulator=="spinnaker"):
        sim.run(nealParameters.SIM_LENGTH)
    elif (nealParameters.simulator=="nest"):
        callbackTimeStepLength = 30
        steps = int(nealParameters.SIM_LENGTH/callbackTimeStepLength)
        for step in range (0, steps):
            currentTime = step*callbackTimeStepLength
            setNestInput(currentTime,parser)
            setNestOutput(currentTime,actionCells)
            print "nest simulation timestep ", currentTime
            sim.run(callbackTimeStepLength)
            time.sleep(0.7)

def connectFinalParseToAction(parser,parseState,actionCells,actionCellNumber):
    parser.parseStateStartsNeuron(parseState,actionCells, actionCellNumber)
    parser.neuronStopsParseState(actionCells, actionCellNumber,parseState)
    

#---main 
init() 
neal = NealCoverFunctions(nealParameters.simulator)
parser = languageToDefine.main()
if  (nealParameters.simulator == 'spinnaker'):
    parser.allocateSpinnakerInputs()
    parser.connectSpinnakerInputsToWords()

if  (nealParameters.simulator == 'nest'):
    agentActionCells.record({'spikes','v'})
elif  (nealParameters.simulator == 'spinnaker'):
    agentActionCells = sim.Population(4,sim.IF_cond_exp,{},
                                      label='actionFromBoard')
    ExternalDevices.activate_live_output_for(agentActionCells,
                    database_notify_port_num=12346)

if  (nealParameters.simulator == 'nest'):
    agentActionCells.record({'spikes','v'})
elif  (nealParameters.simulator == 'spinnaker'):
    agentActionCells.record()
connectFinalParseToAction(parser,3,agentActionCells,0)#turn left
connectFinalParseToAction(parser,6,agentActionCells,2)#step forward

runAgent(parser,agentActionCells)

parser.printResults()
if (nealParameters.simulator == 'nest'):
    agentActionCells.write_data('results/agentAction.pkl','spikes')
elif  (nealParameters.simulator == 'spinnaker'):
    agentActionCells.printSpikes('results/agentAction.sp','spikes')

sim.end()


